Package: easycensus
Title: Quickly Find, Extract, and Marginalize U.S. Census Tables
Version: 1.1.4
Authors@R: 
    person("Cory", "McCartan", email="mccartan@psu.edu", role=c("aut", "cre"))
Description: Extracting desired data using the proper Census variable names can 
    be time-consuming. This package takes the pain out of that process by 
    providing functions to quickly locate variables and download labeled tables 
    from the Census APIs (<https://www.census.gov/data/developers/data-sets.html>).
Depends: R (>= 2.10)
Imports: rlang, vctrs, pillar, dplyr (>= 1.0.0), tidyr (>= 1.0.0),
        stringr, censusapi, cli
Suggests: posterior, testthat (>= 3.0.0)
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
URL: https://corymccartan.com/easycensus/,
        https://github.com/CoryMcCartan/easycensus/,
        http://corymccartan.com/easycensus/
BugReports: https://github.com/CoryMcCartan/easycensus/issues
Config/testthat/edition: 3
Language: en-US
NeedsCompilation: no
Packaged: 2026-05-13 17:09:30 UTC; cmccartan
Author: Cory McCartan [aut, cre]
Maintainer: Cory McCartan <mccartan@psu.edu>
Repository: CRAN
Date/Publication: 2026-05-14 05:10:14 UTC
Built: R 4.5.2; ; 2026-05-14 09:46:52 UTC; unix
