# Generated by roxygen2: do not edit by hand

S3method(format,estimate)
S3method(pillar_shaft,estimate)
S3method(print,cens_table)
S3method(vec_arith,estimate)
S3method(vec_arith.estimate,MISSING)
S3method(vec_arith.estimate,default)
S3method(vec_arith.estimate,estimate)
S3method(vec_arith.estimate,numeric)
S3method(vec_arith.numeric,estimate)
S3method(vec_cast,double.estimate)
S3method(vec_cast,estimate.double)
S3method(vec_cast,estimate.estimate)
S3method(vec_cast,estimate.integer)
S3method(vec_math,estimate)
S3method(vec_proxy_compare,estimate)
S3method(vec_ptype2,double.estimate)
S3method(vec_ptype2,estimate.double)
S3method(vec_ptype2,estimate.estimate)
S3method(vec_ptype2,estimate.integer)
S3method(vec_ptype2,integer.estimate)
S3method(vec_ptype_abbr,estimate)
export(as_estimate)
export(cens_find)
export(cens_find_acs)
export(cens_find_dec)
export(cens_geo)
export(cens_get_acs)
export(cens_get_dec)
export(cens_get_raw)
export(cens_margin_to)
export(cens_parse_tables)
export(est_pct_chg)
export(est_prop)
export(estimate)
export(get_est)
export(get_moe)
export(get_se)
export(is_estimate)
export(tidy_age)
export(tidy_age_bins)
export(tidy_ethnicity)
export(tidy_income_bins)
export(tidy_parens)
export(tidy_race)
export(tidy_race_detailed)
export(tidy_simplify)
export(to_rvar)
import(cli)
import(stringr)
import(vctrs)
importFrom(dplyr,"%>%")
importFrom(dplyr,as_tibble)
importFrom(dplyr,if_else)
importFrom(pillar,pillar_shaft)
importFrom(rlang,.data)
importFrom(rlang,.env)
importFrom(stats,qnorm)
importFrom(stats,rnorm)
importFrom(utils,adist)
importFrom(utils,data)
